/******************************************************************************/
/* Projet.............: Pavage de plan                                        */
/* Fichier............: forme.c                                               */
/* Date de cration...: 08/07/2008                                            */
/* Auteur.............: Florian SURET                                         */
/******************************************************************************/

#include "forme.h"

/* cration d'une forme vierge */
t_forme* creerForme(char lettre){
	/* allocation mmoire */
	t_forme* forme = malloc(sizeof(t_forme));
	
	/* initialisations */
	forme->hauteur = 0;
	forme->largeur = 0;
	forme->lettre = lettre;
	
	return forme;
}

/* affichage d'une forme */
void afficherForme(const t_forme* forme){
	int i,j;
	
	/* parcours de la matrice */
	for (i = 0 ; i < forme->hauteur ; i++) {
		for (j = 0 ; j < forme->largeur ; j++) {
			/* case de la forme */
			if (forme->matrice[i][j]) {
				/* affichage de la lettre */
				printf("%c", forme->lettre);
			} else {
				/* affichage d'un espace */
				printf(" ");
			}
		}
		
		/* saut de ligne */
		printf("\n");
	}
}

/* ajout d'une ligne  la forme */
void ajouterLigne(t_forme* forme, char* ligne){
	int i = 0;
	
	/* parcours de la ligne */
	while (ligne[i] != '\n') {
		/* case de la forme */
		if (ligne[i] == '*') {
			/* ajout de la case */
			forme->matrice[forme->hauteur][i] = 1;
		} else {
			/* ajout d'un blanc*/
			forme->matrice[forme->hauteur][i] = 0;
		}
		
		i++;
	}
	
	/* nouvelle largeur de la forme */
	if (i > forme->largeur) {
		/* largeur de la forme */
		forme->largeur = i;
	}
	
	/* hauteur de la forme */
	forme->hauteur++;
}

/* destruction d'une forme */
void detruireForme(t_forme* forme){
	/* libration mmoire */
	free(forme);
	forme = NULL;
}

/* nombre de cases de la forme */
int nombreCases(const t_forme* forme){
	int i,j;
	
	/* nombre de cases */
	int nombre = 0;
	
	/* parcours de la matrice */
	for (i = 0 ; i < forme->hauteur ; i++) {
		for (j = 0 ; j < forme->largeur ; j++) {
			/* case de la forme */
			if (forme->matrice[i][j]) {
				nombre++;
			}
		}
	}
	
	return nombre;
}

/* rotation d'une forme de 90 */
void tournerForme90(t_forme* forme){
	int i,j, tmp;
	
	/* nouvelle matrice */
	int nouvelle[MAX][MAX];
	
	/* parcours de la matrice */
	for (i = 0 ; i < forme->hauteur ; i++) {
		for (j = 0 ; j < forme->largeur ; j++) {
			/* rotation */
			nouvelle[j][forme->hauteur-i-1] = forme->matrice[i][j];
		}
	}
	
	/* copie de la nouvelle matrice */
	memcpy(forme->matrice, nouvelle, MAX*MAX*sizeof(int));
	
	/* inversion hauteur <-> largeur */
	tmp = forme->hauteur;
	forme->hauteur = forme->largeur;
	forme->largeur = tmp;
}

/* rotation d'une forme de 180 */
void tournerForme180(t_forme* forme){
	int i,j;
	
	/* nouvelle matrice */
	int nouvelle[MAX][MAX];
	
	/* parcours de la matrice */
	for (i = 0 ; i < forme->hauteur ; i++) {
		for (j = 0 ; j < forme->largeur ; j++) {
			/* rotation */
			nouvelle[forme->hauteur-i-1][forme->largeur-j-1] = forme->matrice[i][j];
		}
	}
	
	/* copie de la nouvelle matrice */
	memcpy(forme->matrice, nouvelle, MAX*MAX*sizeof(int));
}

/* quilibre gauche/droite de la forme */
int equilibreGaucheDroite(const t_forme* forme){
	int i,j,gauche,droite;
	
	/* initialisation */
	gauche = 0;
	droite = 0;
	
	/* parcours de la matrice */
	for (i = 0 ; i < forme->hauteur ; i++) {
		for (j = 0 ; j < forme->largeur ; j++) {
			/* case de la forme */
			if (forme->matrice[i][j]) {
				/* gauche/droite */
				if (j < ((double) forme->largeur-1)/2.0) {
					gauche++;
				} else if (j > ((double) forme->largeur-1)/2.0) {
					droite++;
				}
			}
		}
	}
	
	return droite-gauche;
}

/* quilibre haut/bas de la forme */
int equilibreHautBas(const t_forme* forme){
	int i,j,haut,bas;
	
	/* initialisation */
	haut = 0;
	bas = 0;
	
	/* parcours de la matrice */
	for (i = 0 ; i < forme->hauteur ; i++) {
		for (j = 0 ; j < forme->largeur ; j++) {
			/* case de la forme */
			if (forme->matrice[i][j]) {
				/* haut/bas */
				if (i < ((double) forme->hauteur-1)/2.0) {
					haut++;
				} else if (i > ((double) forme->hauteur-1)/2.0) {
					bas++;
				}
			}
		}
	}
	
	return bas-haut;
}
