/******************************************************************************/
/* Projet.............: Pavage de plan                                        */
/* Fichier............: resultat.c                                            */
/* Date de cration...: 08/07/2008                                            */
/* Auteur.............: Florian SURET                                         */
/******************************************************************************/

#include "resultat.h"

/* cration d'un rsultat vierge */
t_resultat* creerResultat(int largeur){
	int i,j;
	
	/* allocation mmoire */
	t_resultat* resultat = malloc(sizeof(t_resultat));
	
	/* initialisation */
	resultat->hauteur = 0;
	resultat->largeur = 0;
	resultat->largeur_max = largeur;
	
	/* allocation mmoire de la matrice*/
	resultat->matrice = calloc(MAX*NB_FORMES, sizeof(char*));
	
	/* parcours des lignes */
	for (i = 0 ; i < NB_FORMES*MAX ; i++) {
		/* allocation mmoire de la matrice */
		resultat->matrice[i] = calloc(largeur, sizeof(char));
		
		/* parcours de la largeur */
		for (j = 0 ; j < largeur ; j++) {
			/* initialisation de la matrice */
			resultat->matrice[i][j] = ' ';
		}
	}
	
	return resultat;
}

/* affichage d'un rsultat */
void afficherResultat(const t_resultat* resultat, FILE* stream) {
	int i,j;
	
	/* parcours de la matrice */
	for (i = 0 ; i < resultat->hauteur ; i++) {
		for (j = 0 ; j < resultat->largeur ; j++) {
			/* affichage de la case */
			fprintf(stream, "%c", resultat->matrice[i][j]);
		}
		
		/* saut de ligne */
		fprintf(stream, "\n");
	}
}

/* test si le positionnement d'une forme est possible */
int estPossible(const t_resultat* resultat, const t_forme* forme, int x, int y){
	int i,j;
	
	/* largeur trop grande */
	if (x+forme->largeur > resultat->largeur_max) {
		return 0;
	}
	
	/* parcours de la forme */
	for (i = 0 ; i < forme->hauteur ; i++) {
		for (j = 0 ; j < forme->largeur ; j++) {
			/* positionnement impossible */
			if (forme->matrice[i][j] && (resultat->matrice[y+i][x+j] != ' ' || x+j >= resultat->largeur_max)) {
				return 0;
			}
		}
	}
	
	return 1;
}

/* positionnement d'une forme */
void positionnerForme(t_resultat* resultat, const t_forme* forme, int x, int y){
	int i,j;
	
	/* parcours de la forme */
	for (i = 0 ; i < forme->hauteur ; i++) {
		for (j = 0 ; j < forme->largeur ; j++) {
			/* positionnement de la case */
			if (forme->matrice[i][j]) {
				resultat->matrice[y+i][x+j] = forme->lettre;
			}
		}
	}
	
	/* nouvelle hauteur */
	if (y+i > resultat->hauteur) {
		resultat->hauteur = y+i;
	}
	
	/* nouvelle largeur */
	if (x+j > resultat->largeur) {
		resultat->largeur = x+j;
	}
}

/* equilibre gauche/droite du resultat */
int equilibreResultat(const t_resultat* resultat){
	int i, j, gauche, droite;
	
	/* initialisation */
	gauche = 0;
	droite = 0;
	
	/* parcours de la matrice */
	for (i = 0 ; i < resultat->hauteur ; i++) {
		for (j = 0 ; j < resultat->largeur ; j++) {
			/* case d'une forme */
			if (resultat->matrice[i][j] != ' ') {
				/* gauche/droite */
				if (j < ((double) resultat->largeur-1)/2.0) {
					gauche++;
				} else if (j > ((double) resultat->largeur-1)/2.0) {
					droite++;
				}
			}
		}
	}
	
	return droite-gauche;
}

/* utilisation d'une forme */
void utiliserForme(t_resultat* resultat, t_forme* forme){
	int x,y, nbRotations, equilibre;
	
	int direction;	/* gauche => 1 */
					/* droite => 0 */
	
	nbRotations = 0;
	
	/* positionnement dans le sens de la largeur */
	if (forme->hauteur > forme->largeur || forme->largeur > resultat->largeur_max) {
		tournerForme90(forme);
	}
	
	/* quilibre haut/bas de la forme */
	if (equilibreHautBas(forme) > 0) {
		tournerForme180(forme);
	}
	
	/* direction gauche/droite */
	equilibre = equilibreGaucheDroite(forme);
	if (equilibre > 0) {
		direction = 1;
		x = resultat->largeur_max-forme->largeur;
	} else if (equilibre < 0) {
		direction = 0;
		x = 0;
	} else {
		/* quilibre gauche/droite du rsultat */
		if (equilibreResultat(resultat) > 0) {
			direction = 1;
			x = resultat->largeur_max-forme->largeur;
		} else {
			direction = 0;
			x = 0;
		}
	}
	
	/* recherche d'une position possible pour la forme */
	while (!estPossible(resultat, forme, x, y)) {
		/* colonne suivante */
		if (direction) {
			x--;
		} else {
			x++;
		}
		
		/* dbut de ligne atteinte */
		if (direction && x <= -1) {
			/* on se place en fin de ligne */
			x = resultat->largeur_max-forme->largeur;
			
			if (nbRotations == 0) {
				/* rotation de 180 de la forme */
				tournerForme180(forme);
				nbRotations++;
				
			} else if (nbRotations == 1) {
				/* rotation de 90 de la forme */
				tournerForme90(forme);
				nbRotations++;
					
			} else if (nbRotations == 2) {
				/* rotation de 180 de la forme */
				tournerForme180(forme);
				nbRotations++;
				
			} else {
				/* rotation de 90 de la forme */
				tournerForme90(forme);
				tournerForme180(forme);
				nbRotations = 0;
				
				/* prochaine ligne */
				y++;
			}
		
		/* fin de ligne atteinte */
		} else if (!direction && x >= resultat->largeur_max-forme->largeur+1) {
			/* on se place en dbut de ligne */
			x = 0;
			
			if (nbRotations == 0) {
				/* rotation de 180 de la forme */
				tournerForme180(forme);
				nbRotations++;
				
			} else if (nbRotations == 1) {
				/* rotation de 90 de la forme */
				tournerForme90(forme);
				nbRotations++;
					
			} else if (nbRotations == 2) {
				/* rotation de 180 de la forme */
				tournerForme180(forme);
				nbRotations++;
				
			} else {
				/* rotation de 90 de la forme */
				tournerForme90(forme);
				tournerForme180(forme);
				nbRotations = 0;
				
				/* prochaine ligne */
				y++;
			}
		}
	}
	
	/* positionnement de la forme */
	positionnerForme(resultat, forme, x, y);
}

/* destruction d'un rsultat */
void detruireResultat(t_resultat* resultat){
	int i;
	
	/* parcours des lignes */
	for (i = 0 ; i < NB_FORMES*MAX ; i++) {
		/* libration mmoire de la matrice */
		free(resultat->matrice[i]);
		resultat->matrice[i] = NULL;
	}
	
	/* libration mmoire de la matrice*/
	free(resultat->matrice);
	resultat->matrice = NULL;
	
	/* libration mmoire du rsultat */
	free(resultat);
	resultat = NULL;
}
